@@ -1,3 +1,27 @@
+/*
+    SDL_ttf:  A companion library to SDL for working with TrueType (tm) fonts
+    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
+
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Library General Public
+    License as published by the Free Software Foundation; either
+    version 2 of the License, or (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    Library General Public License for more details.
+
+    You should have received a copy of the GNU Library General Public
+    License along with this library; if not, write to the Free
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+    Sam Lantinga
+    slouken@libsdl.org
+*/
+
+/* $Id: SDL_ttf.c,v 1.5 2001/12/14 14:40:46 slouken Exp $ */
+
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -123,7 +147,7 @@ int TTF_Init( void )
 	return status;
 }
 
-TTF_Font* TTF_OpenFont( const char *file, int ptsize )
+TTF_Font* TTF_OpenFontIndex( const char *file, int ptsize, long index )
 {
 	TTF_Font* font;
 	FT_Error error;
@@ -144,6 +168,21 @@ TTF_Font* TTF_OpenFont( const char *file, int ptsize )
 		free( font );
 		return NULL;
 	}
+	if ( index != 0 ) {
+		if ( font->face->num_faces > index ) {
+		  	FT_Done_Face( font->face );
+			error = FT_New_Face( library, file, index, &font->face );
+			if( error ) {
+				TTF_SetFTError( "Couldn't get font face", error);
+				free( font );
+				return NULL;
+			}
+		} else {
+			TTF_SetFTError( "No such font face", error);
+			free( font );
+			return NULL;
+		}
+	}
 	face = font->face;
 
 	/* Make sure that our font face is scalable (global metrics) */
@@ -192,6 +231,11 @@ TTF_Font* TTF_OpenFont( const char *file, int ptsize )
 	return font;
 }
 
+TTF_Font* TTF_OpenFont( const char *file, int ptsize )
+{
+	return TTF_OpenFontIndex(file, ptsize, 0);
+}
+
 static void Flush_Glyph( c_glyph* glyph )
 {
 	glyph->stored = 0;
@@ -334,8 +378,8 @@ static FT_Error Load_Glyph( TTF_Font* font, Uint16 ch, c_glyph* cached, int want
 			int soffset = i * src->pitch;
 			int doffset = i * dst->pitch;
 			if ( mono ) {
-				char *srcp = src->buffer + soffset;
-				char *dstp = dst->buffer + doffset;
+				unsigned char *srcp = src->buffer + soffset;
+				unsigned char *dstp = dst->buffer + doffset;
 				int j;
 				for ( j = 0; j < src->width; j += 8 ) {
 					unsigned char ch = *srcp++;
@@ -484,6 +528,26 @@ int TTF_FontLineSkip(TTF_Font *font)
 	return(font->lineskip);
 }
 
+long TTF_FontFaces(TTF_Font *font)
+{
+	return(font->face->num_faces);
+}
+
+int TTF_FontFaceIsFixedWidth(TTF_Font *font)
+{
+	return(FT_IS_FIXED_WIDTH(font->face));
+}
+
+char *TTF_FontFaceFamilyName(TTF_Font *font)
+{
+	return(font->face->family_name);
+}
+
+char *TTF_FontFaceStyleName(TTF_Font *font)
+{
+	return(font->face->style_name);
+}
+
 int TTF_GlyphMetrics(TTF_Font *font, Uint16 ch,
                      int* minx, int* maxx, int* miny, int* maxy, int* advance)
 {
@@ -1097,7 +1161,6 @@ SDL_Surface *TTF_RenderUNICODE_Blended(TTF_Font *font,
 	const Uint16 *ch;
 	Uint8 *src;
 	Uint32 *dst;
-	Uint32 Rmask, Gmask, Bmask, Amask;
 	int row, col;
 	c_glyph *glyph;
 	FT_Error error;
@@ -1109,20 +1172,8 @@ SDL_Surface *TTF_RenderUNICODE_Blended(TTF_Font *font,
 	}
 	height = font->height;
 
-	/* Create the target surface, 32-bit ARGB format */
-	if ( SDL_BYTEORDER == SDL_LIL_ENDIAN ) {
-		Rmask = 0x000000FF;
-		Gmask = 0x0000FF00;
-		Bmask = 0x00FF0000;
-		Amask = 0xFF000000;
-	} else {
-		Rmask = 0xFF000000;
-		Gmask = 0x00FF0000;
-		Bmask = 0x0000FF00;
-		Amask = 0x000000FF;
-	}
 	textbuf = SDL_AllocSurface(SDL_SWSURFACE, width, height, 32,
-	                           Rmask, Gmask, Bmask, Amask);
+                  0x00FF0000, 0x0000FF00, 0x000000FF, 0xFF000000);
 	if ( textbuf == NULL ) {
 		return(NULL);
 	}
@@ -1163,7 +1214,7 @@ SDL_Surface *TTF_RenderUNICODE_Blended(TTF_Font *font,
 			row = (textbuf->h-1) - font->underline_height;
 		}
 		dst = (Uint32 *)textbuf->pixels + row * textbuf->pitch/4;
-		pixel |= Amask;
+		pixel |= 0xFF000000; /* Amask */
 		for ( row=font->underline_height; row>0; --row ) {
 			for ( col=0; col < textbuf->w; ++col ) {
 				dst[col] = pixel;
@@ -1181,7 +1232,6 @@ SDL_Surface *TTF_RenderGlyph_Blended(TTF_Font *font, Uint16 ch, SDL_Color fg)
 	Uint32 pixel;
 	Uint8 *src;
 	Uint32 *dst;
-	Uint32 Rmask, Gmask, Bmask, Amask;
 	int row, col;
 	FT_Error error;
 	c_glyph *glyph;
@@ -1193,21 +1243,9 @@ SDL_Surface *TTF_RenderGlyph_Blended(TTF_Font *font, Uint16 ch, SDL_Color fg)
 	}
 	glyph = font->current;
 
-	/* Create the target surface, 32-bit ARGB format */
-	if ( SDL_BYTEORDER == SDL_LIL_ENDIAN ) {
-		Rmask = 0x000000FF;
-		Gmask = 0x0000FF00;
-		Bmask = 0x00FF0000;
-		Amask = 0xFF000000;
-	} else {
-		Rmask = 0xFF000000;
-		Gmask = 0x00FF0000;
-		Bmask = 0x0000FF00;
-		Amask = 0x000000FF;
-	}
 	textbuf = SDL_CreateRGBSurface(SDL_SWSURFACE,
 	              glyph->pixmap.width, glyph->pixmap.rows, 32,
-	                           Rmask, Gmask, Bmask, Amask);
+                  0x00FF0000, 0x0000FF00, 0x000000FF, 0xFF000000);
 	if ( ! textbuf ) {
 		return(NULL);
 	}
@@ -1230,7 +1268,7 @@ SDL_Surface *TTF_RenderGlyph_Blended(TTF_Font *font, Uint16 ch, SDL_Color fg)
 			row = (textbuf->h-1) - font->underline_height;
 		}
 		dst = (Uint32 *)textbuf->pixels + row * textbuf->pitch/4;
-		pixel |= Amask;
+		pixel |= 0xFF000000; /* Amask */
 		for ( row=font->underline_height; row>0; --row ) {
 			for ( col=0; col < textbuf->w; ++col ) {
 				dst[col] = pixel;
